/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.core;

import ipl.util.Constants;

/**
 * <p>Description: Creates the parse tree.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
class XmlParseTree {

  private ParseTreeNode root;

  public XmlParseTree(String xmlText) {
    int count = 0;
    root = null;
    ParseTreeNode curr = null, temp;
    int start = 0, end, startOfText = 0, endOfText, offset;
    String tag, type;
    String prevChar = "";

    while ( (start = xmlText.indexOf("<", start)) != -1) {
      if ((!xmlText.startsWith(Constants.PARAGRAPH_START_TAG, start)) &&
          (!xmlText.startsWith(Constants.PARAGRAPH_END_TAG, start))) {
        end = xmlText.indexOf(">", start + 1);
        endOfText = start;
        tag = xmlText.substring(start + 1, end);
        type = Constants.NONE;

        // case: end of tag
        if (tag.charAt(0) == '/') {
          if (root == null) {
            System.err.println("PARSE ERROR1");
            System.exit(1);
          } // if
          //      
          tag = tag.substring(1);
          if (curr.getTag().compareTo(tag) != 0) {
            System.err.println("PARSE ERROR2");
            System.exit(1);
          } // if

          //   .     
          curr.setNextText(xmlText.substring(startOfText, endOfText));
          curr.setNextPrevChar(prevChar);
          curr = curr.getParent();
        } // if
        // case: start of tag
        else {
          // get type
          if ( (offset = tag.indexOf(" TYPE")) != -1) {
            type = tag.substring(offset + 1);
            if (type.compareTo("TYPE=\"PERSON\"") == 0) {
              type = Constants.PERSON;
            } // if
            else if (type.compareTo("TYPE=\"ORGANIZATION\"") == 0) {
              type = Constants.ORGANIZATION;
            } // else if
            else if (type.compareTo("TYPE=\"LOCATION\"") == 0) {
              type = Constants.LOCATION;
            } // else if
            else if (type.compareTo("TYPE=\"MONEY\"") == 0) {
              type = Constants.MONEY;
            } // else if
            else if (type.compareTo("TYPE=\"PERCENT\"") == 0) {
              type = Constants.PERCENT;
            } // else if
            else if (type.compareTo("TYPE=\"DATE\"") == 0) {
              type = Constants.DATE;
            } // else if
            else if (type.compareTo("TYPE=\"TIME\"") == 0) {
              type = Constants.TIME;
            } // else if
            tag = tag.substring(0, offset);
          } // if

          //   ,    
          if (root == null) {
            if (count != 0) {
              System.err.println("NOT SUPPORTED TEXT");
              System.exit(1);
            } // if
            root = new ParseTreeNode(tag, type, null);
            curr = root;
            count++;
          } // if
          else {
            temp = new ParseTreeNode(tag, type, curr);
            curr.setNextPointer(temp);
            curr.setNextText(xmlText.substring(startOfText, endOfText));
            curr.setNextPrevChar(prevChar);
            curr = temp;
          } // else
        } // else
        startOfText = end + 1;
      } // if
      if (start == 0) {
        prevChar = "";
      } // if
      else {
        if ( (xmlText.charAt(start - 1) == '\n') ||
             (xmlText.charAt(start - 1) == ' ')) {
          prevChar = Character.toString(xmlText.charAt(start - 1));
        } // if
        else {
          prevChar = "";
        } // else
      } // else
      start++;
    } // while
  } // constructor: XmlParseTree

  protected ParseTreeNode getRoot() {
    return (find(root, Constants.ARTICLE));
  } // getBody

  protected ParseTreeNode find(ParseTreeNode curr, String strTag) {
    if (curr.getTag().equalsIgnoreCase(strTag)) {
      return curr;
    } // if
    else {
      ParseTreeNode temp;
      int i = 0;
      while (i < curr.getPointersSize()) {
        if ((temp = find(curr.getPointer(i), strTag)) != null) {
          return temp;
        } // if
        i++;
      } // while
      return null;
    } // else
  } // find

} // end of class XmlParseTree
