/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc;

import java.io.File;
import java.util.Arrays;

import libsvm.svm;
import libsvm.svm_node;
import libsvm.svm_model;

import ipl.nerc.features.PersonFeaturesManager;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.nerc.lists.ListManager;
import ipl.splitter.SentenceSplitter;
import ipl.util.*;

/**
 * <p>Description: Tag documents.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @version 0.1
 */
public class Tagger {

  public Tagger() {
  } // constructor: Tagger


  //////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////// TAG //////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  public void TAG_FP(String dirName, String modelFileName, ListManager listManager,
                     TimexMatcher matcher, SentenceSplitter splitter) throws Exception {
    File curr;

    System.out.println("please wait...");

    File outdir = new File(dirName + File.separator + "annotated_data");
    outdir.mkdir();

    DirectoryManager dir = new DirectoryManager(dirName);

    svm_model model = svm.svm_load_model(modelFileName);
    int nr_class = svm.svm_get_nr_class(model);
    int categories[] = new int[nr_class];
    svm.svm_get_labels(model, categories);
    double[] prob_estimates = new double[nr_class];

    //      test set
    while ( (curr = dir.getNextFile()) != null) {
      if (curr.isFile()) {
        String attr[];
        int distanceFromStartOfEntity = -1;

        //   
        String text = FileManager.read(curr);
        FeaturesManager fm = new PersonFeaturesManager(text, listManager, matcher, splitter);

        //    tokens  
        while ( (attr = fm.getNextFeatureVector(distanceFromStartOfEntity, null)) != null) {
          double answer, probability;
          if (attr.length != 1) {
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict_probability(model, node, prob_estimates);
            if (categories[0] == -1) {
              probability = prob_estimates[1] - prob_estimates[0];
            } // if
            else {
              probability = prob_estimates[0] - prob_estimates[1];
            } // else
          } // if
          else {
            answer = -1;
            probability = -1;
          } // else
          fm.tokenAt(fm.getOffset() - 1).setPredictedProbability(probability);

          if (answer == 1) {
            fm.tokenAt(fm.getOffset() - 1).setTagType(Constants.PERSON);
          } // if

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while

        FileManager.write(fm.getTokens(), outdir + File.separator + curr.getName());
        fm = null;
      } // if
    } // while

  } // TAG_FP

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  public void TAG_SP(String dirName, String firstPassModelFileName, String secondPassModelFileName,
                   ListManager firstPassListManager, ListManager secondPassListManager,
                   TimexMatcher matcher, SentenceSplitter splitter) throws Exception {
    File curr;
    double margin = 0.8;
    int countTestFP = 0, countTestSP = 0;

    System.out.print("please wait...");

    File outdir = new File(dirName + File.separator + "annotated_data");
    outdir.mkdir();

    DirectoryManager testDir = new DirectoryManager(dirName);

    svm_model model1 = svm.svm_load_model(firstPassModelFileName);
    svm_model model2 = svm.svm_load_model(secondPassModelFileName);

    int nr_class = svm.svm_get_nr_class(model1);
    int categories[] = new int[nr_class];
    svm.svm_get_labels(model1, categories);
    double[] prob_estimates = new double[nr_class];

    //      test set
    while ( (curr = testDir.getNextFile()) != null) {
      if (curr.isFile()) {

        ///////////////////////////////////////////////
        //       //
        ///////////////////////////////////////////////

        String attr[];
        int distanceFromStartOfEntity = -1;
        String temp[] = new String[256];
        int listNum = 0;

        //   
        String text = FileManager.read(curr);
        FeaturesManager vectorsFP = new PersonFeaturesManager(text, firstPassListManager, matcher, splitter);

        //    tokens  
        while ( (attr = vectorsFP.getNextFeatureVector(distanceFromStartOfEntity, null)) != null) {
          double answer, probability;
          if (attr.length != 1) {
            countTestFP++;
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict_probability(model1, node, prob_estimates);
            if (categories[0] == -1) {
              probability = prob_estimates[1] - prob_estimates[0];
            } // if
            else {
              probability = prob_estimates[0] - prob_estimates[1];
            } // else
          } // if
          else {
            answer = -1;
            probability = -1;
          } // else
          vectorsFP.tokenAt(vectorsFP.getOffset() - 1).setPredictedProbability(probability);

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
            String currTok = vectorsFP.tokenAt(vectorsFP.getOffset() - 1).getToken();
            if ((currTok.length() >= 3) && (probability > margin)) {
              temp[listNum++] = currTok.toLowerCase();
            } // if
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
            String currTok = vectorsFP.tokenAt(vectorsFP.getOffset() - 1).getToken();
            if ((currTok.length() >= 3) && (probability > margin)) {
              temp[listNum++] = currTok.toLowerCase();
            } // if
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while

        //      
        String []list = Auxiliary.getSubList(temp, listNum);
        Arrays.sort(list);

        /////////////////////////////////////////////////
        //       //
        /////////////////////////////////////////////////

        distanceFromStartOfEntity = -1;

        FeaturesManager vectorsSP = new PersonFeaturesManager(vectorsFP.getTokens(), secondPassListManager);

        //    tokens  
        while ( (attr = vectorsSP.getNextFeatureVector(distanceFromStartOfEntity, list)) != null) {
          double answer;
          if (attr.length != 1) {
            countTestSP++;
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict(model2, node);
          } // if
          else {
            answer = -1;
          } // else

          if (answer == 1) {
            vectorsSP.tokenAt(vectorsSP.getOffset() - 1).setTagType(Constants.PERSON);
          } // if

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while

        FileManager.write(vectorsSP.getTokens(), outdir + File.separator + curr.getName());

        vectorsFP = null;
        vectorsSP = null;
        list = null;
        temp = null;
      } // if
    } // while

  } // TAG_SP

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  private static void printParametersAndExit(String str) {
    System.err.println(str);
    System.err.println();
    System.err.println();
    System.err.println("java ipl.nerc.Tagger -d <input dir path> [options]");
    System.err.println();
    System.err.println("-d");
    System.err.println("\tThe directory containing text files that you want to process");
    System.err.println("\nOPTIONS");
    System.err.println("=======");
    System.err.println("-fin");
    System.err.println("\tTag using files specialized for financial articles");
    System.err.println("-gen");
    System.err.println("\tTag using files specialized for general newspaper articles");
    System.err.println("-m");
    System.err.println("\tMode:");
    System.err.println("\t\t1. Tag using only the first pass SVM");
    System.err.println("\t\t2. Tag using both the first and second pass SVMs  (default)");
    System.err.println("-1");
    System.err.println("\tThe first pass classifier filename\t(default: \".\\CLASSIFIER_FP\")");
    System.err.println("-2");
    System.err.println("\tThe second pass classifier filename\t(default: \".\\CLASSIFIER_SP\")");
    System.err.println("-l");
    System.err.println("\tThe lists directory\t\t\t(default: \".\\lists\")");
    System.err.println("-t");
    System.err.println("\tThe TIMEX patterns filename\t\t(default: \".\\TIMEX_PATTERNS\")");
    System.err.println("-p");
    System.err.println("\tThe sentence splitter filename\t\t(default: \".\\SENTENCE_SPLITTER\")");
    System.err.println();
    System.exit(1);
  } // printParametersAndExit

  public static void main(String args[]) throws Exception {
    if (args.length < 2) {
      printParametersAndExit("Invalid parameters number!");
    } // if

    int mode = 2;
    String class1 = "CLASSIFIER_FP";
    String class2 = "CLASSIFIER_SP";
    String testdata = "";
    String listsdir = "./lists";
    String patternsfn = "./TIMEX_PATTERNS";
    String separatorfn = "./SENTENCE_SPLITTER";
    for (int i = 0; i < args.length; i += 2) {
      if ((args[i].length() != 2) && (args[i].length() != 4)) {
        printParametersAndExit("Invalid parameters!");
      } // if
      if (args[i].charAt(0) != '-') {
        printParametersAndExit("Invalid parameters!");
      } // if
      char param;
      if (args[i].length() == 4) {
        if (args[i].equalsIgnoreCase("-fin")) {
          param = 'f';
        } // if
        else if (args[i].equalsIgnoreCase("-gen")) {
          param = 'g';
        } // else if
        else {
          param = ' ';
          printParametersAndExit("Invalid parameters!");
        } // else
      } // if
      else {
        param = args[i].charAt(1);
      } // else
      switch (param) {
        case '1':
          class1 = args[i + 1];
          break;
        case '2':
          class2 = args[i + 1];
          break;
        case 'd':
          testdata = args[i + 1];
          break;
        case 'l':
          listsdir = args[i + 1];
          break;
        case 't':
          patternsfn = args[i + 1];
          break;
        case 'p':
          separatorfn = args[i + 1];
          break;
        case 'm':
          mode = Integer.parseInt(args[i + 1]);
          if ((mode != 1) && (mode != 2)) {
            printParametersAndExit("Invalid mode!");
          } // if
          break;
        case 'f':
          class1 = "files_for_financial_articles/CLASSIFIER_FP";
          class2 = "files_for_financial_articles/CLASSIFIER_SP";
          listsdir = "files_for_financial_articles/lists";
          patternsfn = "files_for_financial_articles/TIMEX_PATTERNS";
          separatorfn = "files_for_financial_articles/SENTENCE_SPLITTER";
          i--;
          break;
        case 'g':
          class1 = "files_for_general_articles/CLASSIFIER_FP";
          class2 = "files_for_general_articles/CLASSIFIER_SP";
          listsdir = "files_for_general_articles/lists";
          patternsfn = "files_for_general_articles/TIMEX_PATTERNS";
          separatorfn = "files_for_general_articles/SENTENCE_SPLITTER";
          i--;
          break;
        default:
          printParametersAndExit("Invalid parameters!");
          break;
      } // switch
    } // for

    if (testdata.compareTo("") == 0) {
      printParametersAndExit("You must give the test directory path!");
    } // if
    if (!(new File(testdata)).exists() || !(new File(testdata)).isDirectory()) {
      printParametersAndExit("Invalid test directory path!");
    } // if
    if (!(new File(class1)).exists() || !(new File(class1)).isFile()) {
      printParametersAndExit("Invalid first pass classifier filename!");
    } // if
    if (!(new File(class2)).exists() || !(new File(class2)).isFile()) {
      printParametersAndExit("Invalid second pass classifier filename!");
    } // if
    if (!(new File(listsdir)).exists() || !(new File(listsdir)).isDirectory()) {
      printParametersAndExit("Invalid lists directory!");
    } // if
    if (!(new File(patternsfn)).exists() || !(new File(patternsfn)).isFile()) {
      printParametersAndExit("Invalid TIMEX patterns filename!");
    } // if
    if (!(new File(separatorfn)).exists() || !(new File(patternsfn)).isFile()) {
      printParametersAndExit("Invalid sentence splitter filename!");
    } // if

    TimexMatcher matcher = new TimexMatcher(patternsfn);
    SentenceSplitter splitter = new SentenceSplitter(separatorfn);
    ListManager listManagerFP = ListManager.initListManager("FP", listsdir);

    Tagger tagger = new Tagger();

    if (mode == 1) {
      tagger.TAG_FP(testdata, class1, listManagerFP, matcher, splitter);
    } // if
    else {
      ListManager listManagerSP = ListManager.initListManager("SP", listsdir);
      tagger.TAG_SP(testdata, class1, class2, listManagerFP, listManagerSP, matcher, splitter);
    } // else
  } // end of main

} // end of class Tagger
