/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc;

import java.io.File;
import java.util.Arrays;

import libsvm.svm;
import libsvm.svm_node;
import libsvm.svm_model;

import ipl.nerc.features.PersonFeaturesManager;
import ipl.nerc.features.FeaturesManager;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.nerc.lists.ListManager;
import ipl.splitter.SentenceSplitter;
import ipl.util.*;

/**
 * <p>Description: Tests the performance of a classifier.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @version 0.1
 */
public class Tester {

  public Tester() {
  } // constructor: Tester

  //////////////////////////////////////////////////////////////////////////////
  ////////////////////////////////// TEST //////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  public void TEST_FP(String dirName, String modelFileName, ListManager listManager,
                      TimexMatcher matcher, SentenceSplitter splitter) throws Exception {
    File curr;

    System.out.println("please wait...");

    DirectoryManager testDir = new DirectoryManager(dirName);

    int confusionMatrix[][] = new int[2][2];
    Auxiliary.initConfusionMatrix(confusionMatrix);

    svm_model model = svm.svm_load_model(modelFileName);
    int nr_class = svm.svm_get_nr_class(model);
    int categories[] = new int[nr_class];
    svm.svm_get_labels(model, categories);
    double[] prob_estimates = new double[nr_class];

    //      test set
    while ( (curr = testDir.getNextFile()) != null) {
      if (curr.isFile()) {
        String attr[];
        int distanceFromStartOfEntity = -1;

        //   
        String text = FileManager.read(curr);
        FeaturesManager fm = new PersonFeaturesManager(text, listManager, matcher, splitter);

        //    tokens  
        while ( (attr = fm.getNextFeatureVector(distanceFromStartOfEntity, null)) != null) {
          double answer, rightAnswer, probability;
          if (attr.length != 1) {
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            rightAnswer = Auxiliary.atof(attr[0]);
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict_probability(model, node, prob_estimates);
            if (categories[0] == -1) {
              probability = prob_estimates[1] - prob_estimates[0];
            } // if
            else {
              probability = prob_estimates[0] - prob_estimates[1];
            } // else
          } // if
          else {
            rightAnswer = Auxiliary.atof(attr[0]);
            answer = -1;
            probability = -1;
          } // else
          fm.tokenAt(fm.getOffset() - 1).setPredictedProbability(probability);
          Auxiliary.updateConfusionMatrix(confusionMatrix, rightAnswer, answer);

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while
        fm = null;
      } // if
    } // while

    Auxiliary.showConfusionMatrix(confusionMatrix);
    Auxiliary.showPrecisionRecall(Auxiliary.computePrecisionRecall(confusionMatrix), confusionMatrix.length);
  } // TEST_FP

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  public void TEST_SP(String dirName, String firstPassModelFileName, String secondPassModelFileName,
                   ListManager firstPassListManager, ListManager secondPassListManager,
                   TimexMatcher matcher, SentenceSplitter splitter) throws Exception {
    File curr;
    double margin = 0.8;
    int countTestFP = 0, countTestSP = 0;

    System.out.print("please wait...");

    DirectoryManager testDir = new DirectoryManager(dirName);

    int confusionMatrix[][] = new int[2][2];
    Auxiliary.initConfusionMatrix(confusionMatrix);

    svm_model model1 = svm.svm_load_model(firstPassModelFileName);
    svm_model model2 = svm.svm_load_model(secondPassModelFileName);

    int nr_class = svm.svm_get_nr_class(model1);
    int categories[] = new int[nr_class];
    svm.svm_get_labels(model1, categories);
    double[] prob_estimates = new double[nr_class];

    //      test set
    while ( (curr = testDir.getNextFile()) != null) {
      if (curr.isFile()) {

        ///////////////////////////////////////////////
        //       //
        ///////////////////////////////////////////////

        String attr[];
        int distanceFromStartOfEntity = -1;
        String temp[] = new String[256];
        int listNum = 0;

        //   
        String text = FileManager.read(curr);
        FeaturesManager vectorsFP = new PersonFeaturesManager(text, firstPassListManager, matcher, splitter);

        //    tokens  
        while ( (attr = vectorsFP.getNextFeatureVector(distanceFromStartOfEntity, null)) != null) {
          double answer, probability;
          if (attr.length != 1) {
            countTestFP++;
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict_probability(model1, node, prob_estimates);
            if (categories[0] == -1) {
              probability = prob_estimates[1] - prob_estimates[0];
            } // if
            else {
              probability = prob_estimates[0] - prob_estimates[1];
            } // else
          } // if
          else {
            answer = -1;
            probability = -1;
          } // else
          vectorsFP.tokenAt(vectorsFP.getOffset() - 1).setPredictedProbability(probability);

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
            String currTok = vectorsFP.tokenAt(vectorsFP.getOffset() - 1).getToken();
            if ((currTok.length() >= 3) && (probability > margin)) {
              temp[listNum++] = currTok.toLowerCase();
            } // if
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
            String currTok = vectorsFP.tokenAt(vectorsFP.getOffset() - 1).getToken();
            if ((currTok.length() >= 3) && (probability > margin)) {
              temp[listNum++] = currTok.toLowerCase();
            } // if
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while

        //      
        String []list = Auxiliary.getSubList(temp, listNum);
        Arrays.sort(list);

        /////////////////////////////////////////////////
        //       //
        /////////////////////////////////////////////////

        distanceFromStartOfEntity = -1;

        FeaturesManager vectorsSP = new PersonFeaturesManager(vectorsFP.getTokens(), secondPassListManager);

        //    tokens  
        while ( (attr = vectorsSP.getNextFeatureVector(distanceFromStartOfEntity, list)) != null) {
          double answer, rightAnswer;
          if (attr.length != 1) {
            countTestSP++;
            int numOfAttributes = attr.length - 1;
            svm_node node[] = new svm_node[numOfAttributes];
            rightAnswer = Auxiliary.atof(attr[0]);
            for (int i = 1; i <= numOfAttributes; i++) {
              node[i - 1] = new svm_node();
              node[i - 1].index = i;
              node[i - 1].value = Auxiliary.atof(attr[i]);
            } // for
            // PREDICT
            answer = svm.svm_predict(model2, node);
            Auxiliary.updateConfusionMatrix(confusionMatrix, rightAnswer, answer);
          } // if
          else {
            rightAnswer = Auxiliary.atof(attr[0]);
            answer = -1;
            Auxiliary.updateConfusionMatrix(confusionMatrix, rightAnswer, answer);
          } // else

          //       .
          //         
          // .
          if ((answer == 1) && (distanceFromStartOfEntity == -1)) {
            distanceFromStartOfEntity = 1;
          } // if
          else if (answer == 1) {
            distanceFromStartOfEntity++;
          } // else if
          else if ((answer == -1) && (distanceFromStartOfEntity != -1)) {
            distanceFromStartOfEntity = -1;
          } // else if
        } // while

        vectorsFP = null;
        vectorsSP = null;
        list = null;
        temp = null;
      } // if
    } // while

    Auxiliary.showConfusionMatrix(confusionMatrix);
    Auxiliary.showPrecisionRecall(Auxiliary.computePrecisionRecall(confusionMatrix), confusionMatrix.length);
  } // TEST_SP

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  private static void printParametersAndExit(String str) {
    System.err.println(str);
    System.err.println();
    System.err.println();
    System.err.println("java ipl.nerc.Tester -1 <classifier1> [-2 <classifier2>] -d <test_data> [options]");
    System.err.println();
    System.err.println("-1");
    System.err.println("\tThe first pass classifier output filename");
    System.err.println("-2");
    System.err.println("\tThe second pass classifier output filename");
    System.err.println("-d");
    System.err.println("\tThe test directory path");
    System.err.println("\nOPTIONS");
    System.err.println("=======");
    System.err.println("-m");
    System.err.println("\tMode:");
    System.err.println("\t\t1. Test using only the first pass SVM");
    System.err.println("\t\t2. Test using both the first and second pass SVMs  (default)");
    System.err.println("-l");
    System.err.println("\tThe lists directory\t\t\t(default: \".\\lists\")");
    System.err.println("-t");
    System.err.println("\tThe TIMEX patterns filename\t\t(default: \".\\TIMEX_PATTERNS\")");
    System.err.println("-p");
    System.err.println("\tThe sentence splitter filename\t\t(default: \".\\SENTENCE_SPLITTER\")");
    System.err.println();
    System.exit(1);
  } // printParametersAndExit

  public static void main(String args[]) throws Exception {
    if ((args.length < 4) || (args.length % 2 == 1)) {
      printParametersAndExit("Invalid parameters number!");
    } // if

    int mode = 2;
    String class1 = "";
    String class2 = "";
    String testdata = "";
    String listsdir = "./lists";
    String patternsfn = "./TIMEX_PATTERNS";
    String separatorfn = "./SENTENCE_SPLITTER";
    for (int i = 0; i < args.length; i += 2) {
      if (args[i].length() != 2) {
        printParametersAndExit("Invalid parameters!");
      } // if
      if (args[i].charAt(0) != '-') {
        printParametersAndExit("Invalid parameters!");
      } // if
      char param = args[i].charAt(1);
      switch (param) {
        case '1':
          class1 = args[i + 1];
          break;
        case '2':
          class2 = args[i + 1];
          break;
        case 'd':
          testdata = args[i + 1];
          if (!(new File(testdata)).isDirectory()) {
            printParametersAndExit("Invalid first pass train directory path!");
          } // if
          break;
        case 'l':
          listsdir = args[i + 1];
          break;
        case 't':
          patternsfn = args[i + 1];
          break;
        case 'p':
          separatorfn = args[i + 1];
          break;
        case 'm':
          mode = Integer.parseInt(args[i + 1]);
          if ((mode != 1) && (mode != 2)) {
            printParametersAndExit("Invalid mode!");
          } // if
          break;
        default:
          printParametersAndExit("Invalid parameters!");
          break;
      } // switch
    } // for

    if (class1.compareTo("") == 0) {
      printParametersAndExit("You must give the first pass classifier output filename!");
    } // if
    if (testdata.compareTo("") == 0) {
      printParametersAndExit("You must give the first pass train directory path!");
    } // if
    if (mode == 2) {
      if (class2.compareTo("") == 0) {
        printParametersAndExit("You must give the second pass classifier output filename!");
      } // if
    } // if

    if (!(new File(listsdir)).exists() || !(new File(listsdir)).isDirectory()) {
      printParametersAndExit("Invalid lists directory!");
    } // if
    if (!(new File(patternsfn)).exists() || !(new File(patternsfn)).isFile()) {
      printParametersAndExit("Invalid TIMEX patterns filename!");
    } // if
    if (!(new File(separatorfn)).exists() || !(new File(patternsfn)).isFile()) {
      printParametersAndExit("Invalid sentence splitter filename!");
    } // if

    TimexMatcher matcher = new TimexMatcher(patternsfn);
    SentenceSplitter splitter = new SentenceSplitter(separatorfn);
    ListManager listManagerFP = ListManager.initListManager("FP", listsdir);

    Tester tester = new Tester();

    if (mode == 1) {
      tester.TEST_FP(testdata, class1, listManagerFP, matcher, splitter);
    } // if
    else {
      ListManager listManagerSP = ListManager.initListManager("SP", listsdir);
      tester.TEST_SP(testdata, class1, class2, listManagerFP, listManagerSP, matcher, splitter);
    } // else
  } // end of main

} // end of class Tester
