/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.features;

import ipl.core.Tokenizer;
import ipl.nerc.regex.timex.TimexMatcher;
import ipl.nerc.lists.ListManager;
import ipl.splitter.SentenceSplitter;
import ipl.util.*;

/**
 * <p>Description: Retrieves and manages the feature vectors.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class PersonFeaturesManager extends FeaturesManager {

  public PersonFeaturesManager(String text, ListManager lists, TimexMatcher matcher,
                                    SentenceSplitter splitter) {
    super(text, lists, matcher, splitter);
  } // constructor: PersonFeaturesManager

  public PersonFeaturesManager(Tokenizer tokens, ListManager lists) {
    super(tokens, lists);
  } // constructor: PersonFeaturesManager

  public String[] getNextFeatureVector(int distFromStart, String []firstPassList) {
    if (offset == tokens.size()) {
      return null;
    } // if
    String category = getCategory(Constants.PERSON);

    String currTok = tokens.get(this.offset).getToken();
    if (isInside(Constants.DATE) || isInside(Constants.TIME)) {
      this.offset++;
      return (new String[] {category});
    } // if
    if (distFromStart == -1) {
      if ((isGrWord(this.offset).compareTo("1") != 0) &&
          (isEnWord(this.offset).compareTo("1") != 0)) {
        this.offset++;
        return (new String[] {category});
      } // if
      if ( (firstCapital(this.offset).compareTo("1") != 0) &&
           (isCapitalized(this.offset).compareTo("1") != 0)) {
        this.offset++;
        return (new String[] {category});
      } // if
      if (isNumber(this.offset).compareTo("1") == 0) {//
        this.offset++;//
        return (new String[] {category});//
      } // if//
      if (isGrWord(this.offset).compareTo("1") == 0) {
        if (currTok.endsWith("") || currTok.endsWith("") ||
            currTok.endsWith("") || currTok.endsWith("") ||
            currTok.endsWith("") || currTok.endsWith("") ||
            currTok.endsWith("") || currTok.endsWith("")) {
          this.offset++;
          return (new String[] {category});
        } // if
        if (lists.getSafeList().containsKey(currTok.toLowerCase())) {
          this.offset++;
          return (new String[] {category});
        } // if
      } // if
    } // if

    String[] attributeVector = new String[128];
    int attributeNum = 0;
    attributeVector[attributeNum++] = category;

    String isgrw, isenw, isn, isd, isc, isa, iseop, len, fcap, cap, ending, grSuffix, grPrefix, grEnding, inp;

    // -2
    isgrw = isGrWord(this.offset - 2);
    isenw = isEnWord(this.offset - 2);
    isd = isDot(this.offset - 2);
    isn = isNumber(this.offset - 2);
    isc = isComma (this.offset - 2);
    iseop = isEndOfPeriod(this.offset - 2);
    len = getLengthProperty(this.offset - 2);
    if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
      inp = inPlural(this.offset - 2);
      isa = isAdjective(this.offset - 2);
      fcap = firstCapital(this.offset - 2);
      cap = isCapitalized(this.offset - 2);
      ending = isMasculineNominative(this.offset - 2);
      grSuffix = greekNameSuffixProperty(this.offset - 2);
      grPrefix = greekNamePrefixProperty(this.offset - 2);
      grEnding = greekEnding(this.offset - 2);
    } // if
    else {
      inp = "-1";
      isa = "-1";
      fcap = "-1";
      cap = "-1";
      ending = "-1";
      grSuffix = "-1";
      grPrefix = "-1";
      grEnding = "-1";
    } // else
    attributeVector[attributeNum++] = iseop;
    attributeVector[attributeNum++] = len;
    attributeVector[attributeNum++] = fcap;
    attributeVector[attributeNum++] = cap;
    attributeVector[attributeNum++] = grEnding;

    // -1
    isgrw = isGrWord(this.offset - 1);
    isenw = isEnWord(this.offset - 1);
    isd = isDot(this.offset - 1);
    isn = isNumber(this.offset - 1);
    isc = isComma (this.offset - 1);
    iseop = isEndOfPeriod(this.offset - 1);
    len = getLengthProperty(this.offset - 1);
    if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
      inp = inPlural(this.offset - 1);
      isa = isAdjective(this.offset - 1);
      fcap = firstCapital(this.offset - 1);
      cap = isCapitalized(this.offset - 1);
      ending = isMasculineNominative(this.offset - 1);
      grSuffix = greekNameSuffixProperty(this.offset - 1);
      grPrefix = greekNamePrefixProperty(this.offset - 1);
      grEnding = greekEnding(this.offset - 1);
    } // if
    else {
      inp = "-1";
      isa = "-1";
      fcap = "-1";
      cap = "-1";
      ending = "-1";
      grSuffix = "-1";
      grPrefix = "-1";
      grEnding = "-1";
    } // else
    attributeVector[attributeNum++] = isgrw;
    attributeVector[attributeNum++] = isenw;
    attributeVector[attributeNum++] = isn;
    attributeVector[attributeNum++] = isd;
    attributeVector[attributeNum++] = isc;
    attributeVector[attributeNum++] = iseop;
    attributeVector[attributeNum++] = len;
    attributeVector[attributeNum++] = fcap;
    attributeVector[attributeNum++] = cap;
    attributeVector[attributeNum++] = grEnding;
    attributeVector[attributeNum++] = isInList(lists.getListOfNames(), this.offset - 1);
    if (firstPassList != null) {
      attributeVector[attributeNum++] = getTokenFirstPassProbability(this.offset - 1);
      attributeVector[attributeNum++] = isInList(firstPassList, this.offset - 1);
    } // if

    // 0
    isgrw = isGrWord(this.offset);
    isenw = isEnWord(this.offset);
    isd = isDot(this.offset);
    isn = isNumber(this.offset);
    isc = isComma (this.offset);
    iseop = isEndOfPeriod(this.offset);
    len = getLengthProperty(this.offset);
    if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
      inp = inPlural(this.offset);
      isa = isAdjective(this.offset);
      fcap = firstCapital(this.offset);
      cap = isCapitalized(this.offset);
      ending = isMasculineNominative(this.offset);
      grSuffix = greekNameSuffixProperty(this.offset);
      grPrefix = greekNamePrefixProperty(this.offset);
      grEnding = greekEnding(this.offset);
    } // if
    else {
      inp = "-1";
      isa = "-1";
      fcap = "-1";
      cap = "-1";
      ending = "-1";
      grSuffix = "-1";
      grPrefix = "-1";
      grEnding = "-1";
    } // else
    attributeVector[attributeNum++] = isgrw;
    attributeVector[attributeNum++] = isenw;
    attributeVector[attributeNum++] = inp;
    attributeVector[attributeNum++] = isd;
    attributeVector[attributeNum++] = isc;
    attributeVector[attributeNum++] = isa;
    attributeVector[attributeNum++] = iseop;
    attributeVector[attributeNum++] = len;
    attributeVector[attributeNum++] = fcap;
    attributeVector[attributeNum++] = cap;
    attributeVector[attributeNum++] = ending;
    attributeVector[attributeNum++] = grPrefix;
    attributeVector[attributeNum++] = grSuffix;
    attributeVector[attributeNum++] = grEnding;
    attributeVector[attributeNum++] = isInList(lists.getListOfNames(), this.offset);
    attributeVector[attributeNum++] = insideSubtitle(this.offset);
    if (firstPassList != null) {
      attributeVector[attributeNum++] = getTokenFirstPassProbability(this.offset);
      attributeVector[attributeNum++] = isInList(firstPassList, this.offset);
    } // if

    // +1
    isgrw = isGrWord(this.offset + 1);
    isenw = isEnWord(this.offset + 1);
    isd = isDot(this.offset + 1);
    isn = isNumber(this.offset + 1);
    isc = isComma (this.offset + 1);
    iseop = isEndOfPeriod(this.offset + 1);
    len = getLengthProperty(this.offset + 1);
    if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
      inp = inPlural(this.offset + 1);
      isa = isAdjective(this.offset + 1);
      fcap = firstCapital(this.offset + 1);
      cap = isCapitalized(this.offset + 1);
      ending = isMasculineNominative(this.offset + 1);
      grSuffix = greekNameSuffixProperty(this.offset + 1);
      grPrefix = greekNamePrefixProperty(this.offset + 1);
      grEnding = greekEnding(this.offset + 1);
    } // if
    else {
      inp = "-1";
      isa = "-1";
      fcap = "-1";
      cap = "-1";
      ending = "-1";
      grSuffix = "-1";
      grPrefix = "-1";
      grEnding = "-1";
    } // else
    attributeVector[attributeNum++] = isgrw;
    attributeVector[attributeNum++] = isenw;
    attributeVector[attributeNum++] = isn;
    attributeVector[attributeNum++] = isd;
    attributeVector[attributeNum++] = isc;
    attributeVector[attributeNum++] = iseop;
    attributeVector[attributeNum++] = len;
    attributeVector[attributeNum++] = fcap;
    attributeVector[attributeNum++] = cap;
    attributeVector[attributeNum++] = grPrefix;
    attributeVector[attributeNum++] = grSuffix;
    attributeVector[attributeNum++] = grEnding;
    attributeVector[attributeNum++] = isInList(lists.getListOfNames(), this.offset + 1);
    if (firstPassList != null) {
      attributeVector[attributeNum++] = getTokenFirstPassProbability(this.offset + 1);
      attributeVector[attributeNum++] = isInList(firstPassList, this.offset + 1);
    } // if

    // +2
    isgrw = isGrWord(this.offset + 2);
    isenw = isEnWord(this.offset + 2);
    isd = isDot(this.offset + 2);
    isn = isNumber(this.offset + 2);
    isc = isComma (this.offset + 2);
    iseop = isEndOfPeriod(this.offset + 2);
    len = getLengthProperty(this.offset + 2);
    if ((isgrw.compareTo("1") == 0) || (isenw.compareTo("1") == 0)) {
      inp = inPlural(this.offset + 2);
      isa = isAdjective(this.offset + 2);
      fcap = firstCapital(this.offset + 2);
      cap = isCapitalized(this.offset + 2);
      ending = isMasculineNominative(this.offset + 2);
      grSuffix = greekNameSuffixProperty(this.offset + 2);
      grPrefix = greekNamePrefixProperty(this.offset + 2);
      grEnding = greekEnding(this.offset + 2);
    } // if
    else {
      inp = "-1";
      isa = "-1";
      fcap = "-1";
      cap = "-1";
      ending = "-1";
      grSuffix = "-1";
      grPrefix = "-1";
      grEnding = "-1";
    } // else
    attributeVector[attributeNum++] = isgrw;
    attributeVector[attributeNum++] = isenw;
    attributeVector[attributeNum++] = isn;
    attributeVector[attributeNum++] = iseop;
    attributeVector[attributeNum++] = len;
    attributeVector[attributeNum++] = fcap;
    attributeVector[attributeNum++] = cap;
    attributeVector[attributeNum++] = grPrefix;
    attributeVector[attributeNum++] = grSuffix;
    attributeVector[attributeNum++] = grEnding;
    if (firstPassList != null) {
      attributeVector[attributeNum++] = getTokenFirstPassProbability(this.offset + 2);
      attributeVector[attributeNum++] = isInList(firstPassList, this.offset + 2);
    } // if

    attributeVector[attributeNum++] = isMr();
    attributeVector[attributeNum++] = areMessrs(10);

    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(0), lists.getAvgOfRepeatedWordsList(0), 7);
    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(1), lists.getAvgOfRepeatedWordsList(1), 1);
    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(2), lists.getAvgOfRepeatedWordsList(2), 7);
    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(3), lists.getAvgOfRepeatedWordsList(3), 1);
    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(4), lists.getAvgOfRepeatedWordsList(4), 7);
    attributeVector[attributeNum++] = isInPrevList(lists.getRepeatedWordsList(5), lists.getAvgOfRepeatedWordsList(5), 1);

    attributeVector[attributeNum++] = distLessThan(distFromStart, 1);
    attributeVector[attributeNum++] = distLessThan(distFromStart, 2);
    attributeVector[attributeNum++] = distLessThan(distFromStart, 3);
    attributeVector[attributeNum++] = distLessThan(distFromStart, 6);

    this.offset++;

    return Auxiliary.getSubList(attributeVector, attributeNum);
  } // getNextFeatureVector

} // end of class PersonFeaturesManager
