/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.lists;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.regex.*;

import ipl.core.Tokenizer;
import ipl.util.DirectoryManager;
import ipl.splitter.SentenceSplitter;
import ipl.util.Constants;

/**
 * <p>Description: Contains functions that create the lists.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @version 0.1
 */
public class ListCreator {

  public static void getFreqTokens(String dirName, int less_occurences) {
    File file;
    DirectoryManager dirMan = new DirectoryManager(dirName);

    Hashtable<String, ListNode> htable = new Hashtable<String, ListNode>();

    while ( (file = dirMan.getNextFile()) != null) {
      if (file.isFile()) {
        try {
          byte []buff = new byte[ (int) file.length()];
          FileInputStream fis = new FileInputStream(file);
          fis.read(buff);
          fis.close();
          String text = new String(buff);
          Tokenizer tok = new Tokenizer(text, null, null);
          for (int i = 0; i < tok.size(); i++) {
            if (tok.get(i).getTagType().compareTo(Constants.NONE) == 0) {
              if (htable.containsKey(tok.get(i).getToken().toLowerCase())) {
                htable.get(tok.get(i).getToken(). toLowerCase()).increase();
              } // if
              else {
                htable.put(tok.get(i).getToken().toLowerCase(),
                           new ListNode(tok.get(i).getToken().toLowerCase()));
              } // if
            } // else
          } // for
        } // try
        catch (Exception e) {
          e.printStackTrace();
        } // catch
      } // if
    } // while

    try {
      String outputFileName = "frequentNotNamedEntityWords.txt";
      FileOutputStream fos = new FileOutputStream(outputFileName);
      Enumeration<ListNode> enumeration = htable.elements();
      while (enumeration.hasMoreElements()) {
        ListNode hnode = enumeration.nextElement();
        if (hnode.getCount() >= less_occurences) {
          fos.write( (hnode.getKey() + "\n").getBytes());
        } // if
      } // while
      fos.close();
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch
  } // getFreqTokens

  public static String getPreviousTokens(String dirName, String tagType, String pref,
                                         int less_occurences,
                                         int minWordLength,
                                         int maxWordLength,
                                         int NumOfPreviousWords,
                                         SentenceSplitter splitter,
                                         String outputDirName) {
    byte[] buff;
    String text, outputFileName;
    FileInputStream fis;
    File file;
    DirectoryManager dirMan = new DirectoryManager(dirName);

    Hashtable<String, ListNode> htable = new Hashtable<String, ListNode>();
    Enumeration<ListNode> enumeration;
    Tokenizer tok;

    while ( (file = dirMan.getNextFile()) != null) {
      if (file.isFile()) {
        buff = new byte[ (int) file.length()];
        try {
          fis = new FileInputStream(file);
          fis.read(buff);
          fis.close();
        } // try
        catch (IOException e) {
          e.printStackTrace();
        } // catch
        text = new String(buff);
        tok = new Tokenizer(text, null, splitter);
        for (int i = 0; i < tok.size(); i++) {
          boolean isFirst = false;
          if (tok.get(i).getTagType().compareTo(tagType) == 0) {
            if (i > 0) {
              if (tok.get(i - 1).getTagType().compareTo(tagType) != 0) {
                isFirst = true;
              } // if
            } // if
            else {
              isFirst = true;
            } // else
          } // if
          if (isFirst) {
            int j;
            if ( (i - NumOfPreviousWords) > 0) {
              j = i - NumOfPreviousWords;
            } // if
            else {
              j = 0;
            } // else
            int z = i - 1;
            while ( (z >= j) && (!tok.get(z).getEndOfPeriod())) {
              if (tok.get(z).getTagType().compareTo(Constants.NONE) == 0) {
                if (htable.containsKey(tok.get(z).getToken().toLowerCase())) {
                  htable.get(tok.get(z).getToken().toLowerCase()).increase();
                } // if
                else {
                  htable.put(tok.get(z).getToken().toLowerCase(),
                             new ListNode(tok.get(z).getToken().toLowerCase()));
                } // else
              } // if
              z--;
            } // while
          } // if
        } // for
      } // if
    } // while

    outputFileName = outputDirName + File.separator + pref + tagType +
                     ",lessOcc=" + less_occurences +
                     ",minWordLen=" + minWordLength +
                     ",maxWordLen=" + maxWordLength +
                     ",numPrevWords=" + NumOfPreviousWords;

    try {
      FileOutputStream fos = new FileOutputStream(outputFileName);
      enumeration = htable.elements();
      while (enumeration.hasMoreElements()) {
        ListNode hnode = enumeration.nextElement();
        if ((hnode.getCount() >= less_occurences) && (hnode.getKey().length() >= minWordLength) && (hnode.getKey().length() <= maxWordLength)) {
          fos.write( (hnode.getKey() + " " + hnode.getCount() + "\n").getBytes());
        } // if
      } // while
      fos.close();
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch

    return outputFileName;
  } // getPreviousTokens

  public static String getContainerTokens(String dirName, String[] tagTypes,
                                        int less_occurences, String pref) {
    byte[] buff;
    String text;
    FileInputStream fis;
    File file;
    DirectoryManager dirMan = new DirectoryManager(dirName);

    Hashtable<String, ListNode> htable = new Hashtable<String, ListNode>();
    Enumeration<ListNode> enumeration;
    Tokenizer tok;

    while ( (file = dirMan.getNextFile()) != null) {
      if (file.isFile()) {
        buff = new byte[ (int) file.length()];
        try {
          fis = new FileInputStream(file);
          fis.read(buff);
          fis.close();
        } // try
        catch (IOException e) {
          e.printStackTrace();
        } // catch
        text = new String(buff);
        tok = new Tokenizer(text, null, null);
        for (int i = 0; i < tok.size(); i++) {
          for (int j = 0; j < tagTypes.length; j++) {
            boolean isFirst = false;
            if (tok.get(i).getTagType().compareTo(tagTypes[j]) == 0) {
              if (i > 0) {
                if (tok.get(i - 1).getTagType().compareTo(tagTypes[j]) != 0) {
                  isFirst = true;
                } // if
              } // if
              else {
                isFirst = true;
              } // else
            } // if
            if (isFirst) {
              boolean end = false;
              while (!end) {
                if (!Pattern.matches("[0-9]*", tok.get(i).getToken())) {
                  if (htable.containsKey(tok.get(i).getToken().toLowerCase())) {
                    htable.get(tok.get(i).getToken().toLowerCase()).increase();
                  } // if
                  else {
                    htable.put(tok.get(i).getToken().toLowerCase(),
                               new ListNode(tok.get(i).getToken().toLowerCase()));
                  } // else
                } // if
                boolean isLast = false;
                if (tok.get(i).getTagType().compareTo(tagTypes[j]) == 0) {
                  if (i < tok.size() - 1) {
                    if (tok.get(i + 1).getTagType().compareTo(tagTypes[j]) != 0) {
                      isLast = true;
                    } // if
                  } // if
                  else {
                    isLast = true;
                  } // else
                } // if
                if (isLast) {
                  end = true;
                } // if
                else {
                  i++;
                } // else
              } // while
            } // if
          } // for
        } // for
      } // if
    } // while

    String fileName = "../lists/" + pref + tagTypes[0];
    for (int j = 1; j < tagTypes.length; j++) {
      fileName += "_" + tagTypes[j];
    } // for
    fileName += ",lessOcc=" + less_occurences;

    try {
      FileOutputStream fos = new FileOutputStream(fileName);
      enumeration = htable.elements();
      while (enumeration.hasMoreElements()) {
        ListNode hnode = enumeration.nextElement();
        if (hnode.getCount() >= less_occurences) {
          fos.write((hnode.getKey() + " " + hnode.getCount() + "\n").getBytes());
        } // if
      } // while
      fos.close();
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch

    return fileName;
  } // getContainerTokens

} // end of class ListCreator
