/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.nerc.regex.timex;

import java.io.*;
import java.util.StringTokenizer;

import ipl.nerc.regex.PatternNode;
import ipl.util.*;
import ipl.core.Tokenizer;

/**
 * <p>Description: Marks documents with temporal expressions tags using already
 * created patterns. Moreover, is used to evaluate the temporal expression
 * classifier.</p>
 *
 * <p>Copyright (c) 2005</p>
 * @version 0.1
 */
public class TimexMatcher extends TimexRegex {

  public TimexMatcher(String args[]) {
    super();

    if (args.length == 0) {
      printParametersAndExit("Invalid parameters");
    } // if

    String f = "";
    String d = "";
    for (int i = 0; i < args.length; i += 2) {
      char param = args[i].charAt(1);
      switch (param) {
        case 'f':
          if (args.length <= i + 1) {
            printParametersAndExit("Invalid filename");
          } // if
          f = args[i + 1];
          if (!(new File(f)).isFile()){
            printParametersAndExit("Invalid filename");
          } // if
          break;
        case 'd':
          if (args.length <= i + 1) {
            printParametersAndExit("Invalid directory name");
          } // if
          d = args[i + 1];
          if (!(new File(d)).isDirectory()) {
            printParametersAndExit("Invalid directory name");
          } // if
          break;
        default:
          printParametersAndExit("Invalid parameters");
          break;
      } // switch
    } // for

    if (f.length() == 0) {
      printParametersAndExit("You must give the patterns filename (-f)");
    } // if
    if (d.length() == 0) {
      printParametersAndExit("You must give the test directory name (-d)");
    } // if

    loadFile(f);
    sort();
    //    args[0]        
    // .
    args[0] = d;
  } // constructor: TimexMatcher

  public TimexMatcher(String fileName) {
    super();
    loadFile(fileName);
    sort();
  } // constructor: TimexMatcher

  public TimexMatcher() {
    super();
  } // constructor: TimexMatcher

  private void printParametersAndExit(String str) {
    System.err.println(str);
    System.err.println();
    System.err.println();
    System.err.println("java ipl.nerc.regex.timex.TimexMatcher -f <patterns> -d <test>");
    System.err.println();
    System.err.println("-f");
    System.err.println("\tThe filename which contains the already produced patterns");
    System.err.println("-d");
    System.err.println("\tThe directory name of test data");
    System.err.println();
    System.exit(1);
  } // printParametersAndExit

  private void loadFile(String fileName) {
    String text;
    File file = new File(fileName);
    byte[] buff = new byte[(int) file.length()];
    try {
      FileInputStream fis = new FileInputStream(file);
      fis.read(buff);
      fis.close();
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch

    text = new String(buff);
    text = Auxiliary.removeNoisySpaces(text);
    StringTokenizer strTok = new StringTokenizer(text, "\n");
    while (strTok.hasMoreTokens()) {
      int count = Auxiliary.atoi(strTok.nextToken());
      StringTokenizer expr = new StringTokenizer(strTok.nextToken(), " ");
      String[] expression = new String[expr.countTokens()];
      int i = 0;
      while (expr.hasMoreElements()) {
        expression[i++] = expr.nextToken();
      } // while
      regexList.add(new PatternNode(expression, count));

      expr = null;
    } // while
    strTok = null;
  } // loadFile

  public void setDateTimeTags(Tokenizer tokenizer) {
    int size = tokenizer.size();
    int end;
    for (int i = 0; i < size; i++) {
      if ((end = match(tokenizer, i)) != -1) {
        for (int j = i; j <= end; j++) {
          tokenizer.get(j).setTagType(Constants.DATE);
        } // for
        i = end;
      } // if
      else if ((tokenizer.get(i).getTagType().compareTo(Constants.DATE) == 0) ||
               (tokenizer.get(i).getTagType().compareTo(Constants.TIME) == 0)) {
        tokenizer.get(i).setTagType(Constants.NONE);
      } // else if
    } // for
  } // setDateTimeTags

/*  public void test(String dirName) {
    long startTime, endTime;
    startTime = System.currentTimeMillis();
    int confusionMatrix[][] = new int[2][2];
    Auxiliary.initConfusionMatrix(confusionMatrix);
    DirectoryManager dirManager = new DirectoryManager(dirName);
    File curr;

    try {
      while ((curr = dirManager.getNextFile()) != null) {
        if (curr.isFile()) {
          byte[] buff = new byte[ (int) curr.length()];
          FileInputStream fis = new FileInputStream(curr);
          fis.read(buff);
          fis.close();

          String text = new String(buff);
          Tokenizer tokenizer = new Tokenizer(text, null, null);
          int size = tokenizer.size();
          int end;
          for (int i = 0; i < size; i++) {
            if ( (end = match(tokenizer, i)) != -1) {
              for (int j = i; j <= end; j++) {
                double rightAnswer;
                double answer = 1.0;
                if ((tokenizer.get(j).getTagType().compareTo(Constants.DATE) == 0) ||
                    (tokenizer.get(j).getTagType().compareTo(Constants.TIME) == 0)) {
                  rightAnswer = 1.0;
                } // if
                else {
                  rightAnswer = -1.0;
                } // else
                Auxiliary.updateConfusionMatrix(confusionMatrix, rightAnswer, answer);
              } // for
              i = end;
            } // if
            else {
              double rightAnswer;
              double answer = -1.0;
              if ((tokenizer.get(i).getTagType().compareTo(Constants.DATE) == 0) ||
                  (tokenizer.get(i).getTagType().compareTo(Constants.TIME) == 0)) {
                rightAnswer = 1.0;
              } // if
              else {
                rightAnswer = -1.0;
              } // else
              Auxiliary.updateConfusionMatrix(confusionMatrix, rightAnswer, answer);
            } // else
          } // for
        } // if
      } // while
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch

    endTime = System.currentTimeMillis();
    System.err.println(((endTime - startTime) / 1000) + " seconds\n");
    Auxiliary.showConfusionMatrix(confusionMatrix);
    Auxiliary.showPrecisionRecall(Auxiliary.computePrecisionRecall(confusionMatrix), confusionMatrix.length);
  } // test

  public static void main(String args[]) throws IOException {
    TimexMatcher matcher = new TimexMatcher(args);
    String dirName = args[0];
    matcher.test(dirName);
  } // end of main*/

} // end of class TimexMatcher
