/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.splitter;

import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;

import ipl.core.Tokenizer;

public class SentenceSplitter {

  svm_model classifier;
  SplitterFeaturesManager featmanager;

  public SentenceSplitter(String classifierfn) {
    try {
      classifier = svm.svm_load_model(classifierfn);
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch
  } // contructor: SentenceSplitter

  public void setTokenizer(Tokenizer tokenizer) {
    featmanager = null;
    featmanager = new SplitterFeaturesManager(tokenizer);
  } // setTokenizer

  public void setSentenceTags() {
    if (classifier == null) {
      System.err.println("SentenceSplitter.setSentenceTags: Error");
      System.exit(1);
    } // if
    try {
      int offset = 0;
      String features[];
      while ((features = featmanager.getNextFeaturesVector()) != null) {
        offset = featmanager.getOffset();
        int numOfAttributes = features.length - 1;
        svm_node node[] = new svm_node[numOfAttributes];
        for (int i = 1; i <= numOfAttributes; i++) {
          node[i - 1] = new svm_node();
          node[i - 1].index = i;
          node[i - 1].value = Double.parseDouble(features[i]);
        } // for
        if (svm.svm_predict(classifier, node) >= 0) {
          featmanager.getTokens().get(offset).setEndOfPeriod(true);
        } // if
      } // while
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch
  } // setSentenceTags

} // end of class SentenceSplitter
