/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.splitter;

import java.io.*;

import ipl.util.*;

/**
 * <p><b>We use libSVM package.</b><br>
 * Please read the LIBSVM-COPYRIGHT file.<br>
 * </p>
 * <p>Description: Trains a classifier for period identification.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @version 0.1
 * @see <a href="http://www.csie.ntu.edu.tw/~cjlin/libsvm">libSVM</a>
 */
public class SplitterTrainer {

  public SplitterTrainer() {
  } // constructor: SplitterTrainer

  public void createVectorsFile(String dirName, String outFilename) {
    if (!(new File(dirName)).isDirectory()) {
      System.err.println("\nNot valid directory path");
      System.exit(1);
    } // if

    File curr;
    String features[];
    DirectoryManager trainDir = new DirectoryManager(dirName);

    FeaturesFileManager file = new FeaturesFileManager(outFilename);
    while ((curr = trainDir.getNextFile()) != null) {
      String text = FileManager.read(curr);
      SplitterFeaturesManager featmanager = new SplitterFeaturesManager(text);
      while ((features = featmanager.getNextFeaturesVector()) != null) {
        file.setData(features);
      } // while
    } // while
    file.close();
  } // createVectorsFile

  private static void printParametersAndExit(String str) {
    System.err.println(str);
    System.err.println();
    System.err.println();
    System.err.println("java ipl.splitter.SplitterTrainer -d <train> -o <classifier> [-c <c> -g <gamma>]");
    System.err.println();
    System.err.println("-d");
    System.err.println("\tThe train directory path");
    System.err.println("-o");
    System.err.println("\tThe classifier output filename");
    System.err.println("-c");
    System.err.println("\tThe parameter c of svm\t\t(default: 16)");
    System.err.println("-g");
    System.err.println("\tThe parameter gamma of svm\t(default: 0.125)");
    System.err.println();
    System.exit(1);
  } // printParametersAndExit

  public static void main(String args[]) {
    if ((args.length < 4) || (args.length % 2 == 1)) {
      printParametersAndExit("Invalid parameters number!");
    } // if

    String o = "";
    String d = "";
    double c = 16.0;
    double g = 0.125;
    for (int i = 0; i < args.length; i += 2) {
      if (args[i].length() != 2) {
        printParametersAndExit("Invalid parameters!");
      } // if
      if (args[i].charAt(0) != '-') {
        printParametersAndExit("Invalid parameters!");
      } // if
      char param = args[i].charAt(1);
      switch (param) {
        case 'o':
          o = args[i + 1];
          break;
        case 'd':
          d = args[i + 1];
          if (!(new File(d)).isDirectory()) {
            printParametersAndExit("Invalid directory path!");
          } // if
          break;
        case 'c':
          c = Double.parseDouble(args[i + 1]);
          break;
        case 'g':
          g = Double.parseDouble(args[i + 1]);
          break;
        default:
          printParametersAndExit("Invalid parameters!");
          break;
      } // switch
    } // for

    if (o.compareTo("") == 0) {
      System.err.println("You must give the classifier output filename!");
      System.exit(1);
    } // if
    if (d.compareTo("") == 0) {
      System.err.println("You must give the train directory path!");
      System.exit(1);
    } // if

    SplitterTrainer trainer = new SplitterTrainer();
    String vectors = "TRAIN_VECTORS";
    trainer.createVectorsFile(d, vectors);
    NativeTrainer.train(vectors.toCharArray(), vectors.length(),
                        o.toCharArray(), o.length(), 17, false, c, g);
    (new File(vectors)).deleteOnExit();
  } // end of main

} // end of class SplitterTrainer
