/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;

public class AnnotationInterface extends JFrame {
  private File currFile;
  final private int startHorizSize = 980;
  final private int diffHoriz = 8;
  final private int startVertSize = 735;
  final private int diffVert = 48;

  MouseListener popupListener;
  JScrollPane myScrollPane = new JScrollPane();
  JEditorPane myEditorPane = new JEditorPane();

  // ------ MENU ------
  JMenuBar myMenuBar = new JMenuBar();

  JMenu MenuFile = new JMenu();
  JMenuItem MIOpen = new JMenuItem();
  JMenuItem MISave = new JMenuItem();
  JMenuItem MIExit = new JMenuItem();

  JMenu MenuEdit = new JMenu();
  JMenuItem MITagAsPersonName = new JMenuItem();
  JMenuItem MITagAsOrganization = new JMenuItem();
  JMenuItem MITagAsLocation = new JMenuItem();
  JMenuItem MITagAsMoney = new JMenuItem();
  JMenuItem MITagAsPercent = new JMenuItem();
  JMenuItem MITagAsDate = new JMenuItem();
  JMenuItem MITagAsTime = new JMenuItem();

  // ------ POPUP MENU ------

  JPopupMenu myPopupMenu = new JPopupMenu();
  JMenuItem MIPersonName = new JMenuItem();
  JMenuItem MIOrganization = new JMenuItem();
  JMenuItem MILocation = new JMenuItem();
  JMenuItem MIMoney = new JMenuItem();
  JMenuItem MIPercent = new JMenuItem();
  JMenuItem MIDate = new JMenuItem();
  JMenuItem MITime = new JMenuItem();

  public AnnotationInterface() {
    currFile = null;
    try {
      jbInit();
    } // try
    catch (Exception ex) {
      ex.printStackTrace();
    } // catch
  } // AnnotationInterface

  void jbInit() throws Exception {
    this.getContentPane().setLayout(null);
    this.setDefaultCloseOperation(EXIT_ON_CLOSE);
    this.setResizable(false);
    this.setSize(new Dimension(startHorizSize, startVertSize));
    this.setTitle("GREEK NERC");
    this.addComponentListener(new NERCInterface_this_componentAdapter(this));
    myScrollPane.setBounds(new Rectangle(0, 0, startHorizSize - diffHoriz, startVertSize - diffVert));
    myEditorPane.setSelectionEnd(0);
    myEditorPane.setFont(new java.awt.Font("Courier New", 0, 14));
    this.getContentPane().add(myScrollPane, null);
    myScrollPane.getViewport().add(myEditorPane, null);
    popupListener = new PopupListener(myPopupMenu);
    myPopupMenu.addMouseListener(popupListener);
    myEditorPane.addMouseListener(popupListener);

    // ------ MENU ------
    this.setJMenuBar(myMenuBar);
    myMenuBar.setAlignmentY( (float) 0.0);

    MenuFile.setText("File");
    MIOpen.setText("Open");
    MIOpen.addActionListener(new NERCInterface_MIOpen_actionAdapter(this));
    MISave.setText("Save");
    MISave.addActionListener(new NERCInterface_MISave_actionAdapter(this));
    MIExit.setText("Exit");
    MIExit.addActionListener(new NERCInterface_MIExit_actionAdapter(this));
    myMenuBar.add(MenuFile);
    MenuFile.add(MIOpen);
    MenuFile.add(MISave);
    MenuFile.addSeparator();
    MenuFile.add(MIExit);

    MenuEdit.setText("Edit");
    MITagAsPersonName.setText("Tag As Person Name");
    MITagAsPersonName.addActionListener(new NERCInterface_MITagAsPersonName_actionAdapter(this));
    MITagAsOrganization.setText("Tag As Organization");
    MITagAsOrganization.addActionListener(new NERCInterface_MITagAsOrganization_actionAdapter(this));
    MITagAsLocation.setText("Tag As Location");
    MITagAsLocation.addActionListener(new NERCInterface_MITagAsLocation_actionAdapter(this));
    MITagAsMoney.setText("Tag As Money");
    MITagAsMoney.addActionListener(new NERCInterface_MITagAsMoney_actionAdapter(this));
    MITagAsPercent.setText("Tag As Percent");
    MITagAsPercent.addActionListener(new NERCInterface_MITagAsPercent_actionAdapter(this));
    MITagAsDate.setText("Tag As Date");
    MITagAsDate.addActionListener(new NERCInterface_MITagAsDate_actionAdapter(this));
    MITagAsTime.setText("Tag As Time");
    MITagAsTime.addActionListener(new NERCInterface_MITagAsTime_actionAdapter(this));
    myMenuBar.add(MenuEdit);
    MenuEdit.add(MITagAsPersonName);
    MenuEdit.add(MITagAsOrganization);
    MenuEdit.add(MITagAsLocation);
    MenuEdit.add(MITagAsMoney);
    MenuEdit.add(MITagAsPercent);
    MenuEdit.add(MITagAsDate);
    MenuEdit.add(MITagAsTime);

    // ------ POPUP MENU ------
    MIPersonName.setText("Person Name");
    MIPersonName.addActionListener(new NERCInterface_MIPersonName_actionAdapter(this));
    MIOrganization.setText("Organization");
    MIOrganization.addActionListener(new NERCInterface_MIOrganization_actionAdapter(this));
    MILocation.setText("Location");
    MILocation.addActionListener(new NERCInterface_MILocation_actionAdapter(this));
    MIMoney.setText("Money");
    MIMoney.addActionListener(new NERCInterface_MIMoney_actionAdapter(this));
    MIPercent.setText("Percent");
    MIPercent.addActionListener(new NERCInterface_MIPercent_actionAdapter(this));
    MIDate.setText("Date");
    MIDate.addActionListener(new NERCInterface_MIDate_actionAdapter(this));
    MITime.setText("Time");
    MITime.addActionListener(new NERCInterface_MITime_actionAdapter(this));
    myPopupMenu.add(MIPersonName);
    myPopupMenu.add(MIOrganization);
    myPopupMenu.add(MILocation);
    myPopupMenu.add(MIMoney);
    myPopupMenu.add(MIPercent);
    myPopupMenu.add(MIDate);
    myPopupMenu.add(MITime);
  } // jbInit

  public static void main(String[] args) throws IOException {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch

    new AnnotationInterface().setVisible(true);
  } // end of main

  private void tagAs(String startTag, String endTag) {
    int start = myEditorPane.getSelectionStart();
    int end = myEditorPane.getSelectionEnd();
    String text = myEditorPane.getText();
    String pref = text.substring(0, start);
    String suff = text.substring(end, text.length());
    String selectedText = text.substring(start, end);
    text = pref + startTag + selectedText + endTag + suff;
    myEditorPane.setText(text);
    myEditorPane.setSelectionStart(start);
    myEditorPane.setSelectionEnd(startTag.length() + end + endTag.length());
    myEditorPane.setSelectionColor(Color.red);
  } // tagAs

  void MIOpen_actionPerformed(ActionEvent e) {
    MyFileChooser chooser = new MyFileChooser();
    chooser.setDialogTitle("Select file");
    chooser.setApproveButtonText("Select");
    int returnVal = chooser.showOpenDialog(this);
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      try {
        this.setTitle("GREEK NERC - " + chooser.getSelectedFile().getName());
        currFile = chooser.getSelectedFile();
        FileInputStream fis = new FileInputStream(currFile);
        byte buff[] = new byte[ (int) currFile.length()];
        fis.read(buff);
        String text = new String(buff);
        text = Auxiliary.removeNoisySpaces(text);
        myEditorPane.setText(text);
        fis.close();
      } // try
      catch (Exception ex) {
        ex.printStackTrace();
      } // catch
    } // if
  } // MIOpen_actionPerformed

  void this_componentResized(ComponentEvent e) {
    myScrollPane.setSize(getWidth() - diffHoriz, getHeight() - diffVert);
  } // this_componentResized

  void MISave_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    try {
      String text = myEditorPane.getText();
      FileOutputStream fos = new FileOutputStream(currFile);
      byte buff[] = new byte[text.length()];
      buff = text.getBytes();
      fos.write(buff);
      fos.close();
    } // try
    catch (Exception ex) {
      ex.printStackTrace();
    } // catch
  } // MISave_actionPerformed

  void MIExit_actionPerformed(ActionEvent e) {
    System.exit(0);
  } // MIExit_actionPerformed

  void MITagAsPersonName_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"PERSON\">", "</ENAMEX>");
  } // MITagAsPersonName_actionPerformed

  void MIPersonName_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"PERSON\">", "</ENAMEX>");
  } // MIPersonName_actionPerformed

  void MITagAsOrganization_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"ORGANIZATION\">", "</ENAMEX>");
  } // MITagAsOrganization_actionPerformed

  void MIOrganization_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"ORGANIZATION\">", "</ENAMEX>");
  } // MIOrganization_actionPerformed

  void MITagAsLocation_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"LOCATION\">", "</ENAMEX>");
  } // MITagAsLocation_actionPerformed

  void MILocation_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<ENAMEX TYPE=\"LOCATION\">", "</ENAMEX>");
  } // MILocation_actionPerformed

  void MITagAsMoney_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<NUMEX TYPE=\"MONEY\">", "</NUMEX>");
  } // MITagAsMoney_actionPerformed

  void MIMoney_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<NUMEX TYPE=\"MONEY\">", "</NUMEX>");
  } // MIMoney_actionPerformed

  void MITagAsPercent_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<NUMEX TYPE=\"PERCENT\">", "</NUMEX>");
  } // MITagAsPercent_actionPerformed

  void MIPercent_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<NUMEX TYPE=\"PERCENT\">", "</NUMEX>");
  } // MIPercent_actionPerformed

  void MITagAsDate_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<TIMEX TYPE=\"DATE\">", "</TIMEX>");
  } // MITagAsDate_actionPerformed

  void MIDate_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<TIMEX TYPE=\"DATE\">", "</TIMEX>");
  } // MIDate_actionPerformed

  void MITagAsTime_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<TIMEX TYPE=\"TIME\">", "</TIMEX>");
  } // MITagAsTime_actionPerformed

  void MITime_actionPerformed(ActionEvent e) {
    if (currFile == null) {
      return;
    } // if
    tagAs("<TIMEX TYPE=\"TIME\">", "</TIMEX>");
  } // MITime_actionPerformed

} // end of class AnnotationInterface

class NERCInterface_MIOpen_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIOpen_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIOpen_actionPerformed(e);
  }
}

class NERCInterface_this_componentAdapter
    extends java.awt.event.ComponentAdapter {
  AnnotationInterface adaptee;

  NERCInterface_this_componentAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void componentResized(ComponentEvent e) {
    adaptee.this_componentResized(e);
  }
}

class NERCInterface_MISave_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MISave_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MISave_actionPerformed(e);
  }
}

class NERCInterface_MITagAsPersonName_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsPersonName_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsPersonName_actionPerformed(e);
  }
}

class NERCInterface_MIExit_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIExit_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIExit_actionPerformed(e);
  }
}

class NERCInterface_MIPersonName_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIPersonName_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIPersonName_actionPerformed(e);
  }
}

class PopupListener
    extends MouseAdapter {
  JPopupMenu popup;

  PopupListener(JPopupMenu popupMenu) {
    popup = popupMenu;
  }

  public void mousePressed(MouseEvent e) {
    maybeShowPopup(e);
  }

  public void mouseReleased(MouseEvent e) {
    maybeShowPopup(e);
  }

  private void maybeShowPopup(MouseEvent e) {
    if (e.isPopupTrigger()) {
      popup.show(e.getComponent(),
                 e.getX(), e.getY());
    }
  }
}

class NERCInterface_MITagAsOrganization_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsOrganization_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsOrganization_actionPerformed(e);
  }
}

class NERCInterface_MIOrganization_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIOrganization_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIOrganization_actionPerformed(e);
  }
}

class NERCInterface_MITagAsLocation_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsLocation_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsLocation_actionPerformed(e);
  }
}

class NERCInterface_MILocation_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MILocation_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MILocation_actionPerformed(e);
  }
}

class NERCInterface_MITagAsMoney_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsMoney_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsMoney_actionPerformed(e);
  }
}

class NERCInterface_MIMoney_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIMoney_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIMoney_actionPerformed(e);
  }
}

class NERCInterface_MITagAsPercent_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsPercent_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsPercent_actionPerformed(e);
  }
}

class NERCInterface_MIPercent_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIPercent_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIPercent_actionPerformed(e);
  }
}

class NERCInterface_MITagAsDate_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsDate_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsDate_actionPerformed(e);
  }
}

class NERCInterface_MIDate_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MIDate_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MIDate_actionPerformed(e);
  }
}

class NERCInterface_MITime_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITime_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITime_actionPerformed(e);
  }
}

class NERCInterface_MITagAsTime_actionAdapter
    implements java.awt.event.ActionListener {
  AnnotationInterface adaptee;

  NERCInterface_MITagAsTime_actionAdapter(AnnotationInterface adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.MITagAsTime_actionPerformed(e);
  }
}
