/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Arrays;

/**
 * <p>Description: Contains auxiliary functions.</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @version 0.1
 */
public class Auxiliary {

  public static String removeNoisySpaces(String text) {
    String tmp;
    int start, end;

    for (int i = 0; i < text.length(); i++) {
      if ( (Character.isWhitespace(text.charAt(i))) && (text.charAt(i) != ' ') &&
           (text.charAt(i) != '\n')) {
        if (i == 0) {
          tmp = text.substring(1, text.length());
        } // if
        else {
          start = i;
          end = start + 1;
          tmp = text.substring(0, start);
          tmp = tmp.concat(text.substring(end, text.length()));
        } // else
        text = tmp;
      } // if
    } // for
    return text;
  } // removeNoisySpaces

  public static String[] getSubList(String[] temp, int end) {
    String list[] = new String[end];
    for (int i = 0; i < end; i++) {
      list[i] = temp[i];
    } // for
    return list;
  } // getSubList

  public static double atof(String s) {
    return Double.valueOf(s).doubleValue();
  } // atof

  public static int atoi(String s) {
    return Integer.parseInt(s);
  } // atoi

  public static void initConfusionMatrix(int [][] confusionMatrix) {
    int size = confusionMatrix.length;
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
        confusionMatrix[i][j] = 0;
      } // for
    } // for
  } // initConfusionMatrix

  public static void updateConfusionMatrix(int [][] confusionMatrix, double rightAnswer, double answer) {
    if (rightAnswer == -1) {
      if (answer == -1) {
        confusionMatrix[0][0]++;
      } //  if
      else if (answer == 1) {
        confusionMatrix[0][1]++;
      } // else if
      else {
        System.err.println("updateConfusionMatrix: ERROR1");
        System.exit(1);
      } // else
    } // if
    else if (rightAnswer == 1) {
      if (answer == -1) {
        confusionMatrix[1][0]++;
      } //  if
      else if (answer == 1) {
        confusionMatrix[1][1]++;
      } // else if
      else {
        System.err.println("updateConfusionMatrix: ERROR2");
        System.exit(1);
      } // else
    } // else if
    else {
      System.err.println("updateConfusionMatrix: ERROR3");
      System.exit(1);
    } // else
  } // updateConfusionMatrix

  public static double round(double f, int dec) {
    String strDouble = Double.toString(f);
    if (strDouble.length() > (dec + 2)) {
      strDouble = strDouble.substring(0, strDouble.indexOf(".") + dec + 1);
    } // if
    return Double.parseDouble(strDouble);
  } // round

  public static double[][] computePrecisionRecall(int [][] confusionMatrix) {
    int size = confusionMatrix.length;

    double [][] prec_rec = new double[size][3];
    for (int i = 0; i < size; i++) {
      for (int j = 0; j < 3; j++) {
        prec_rec[i][j] = 0;
      } // for
    } // for

    for (int i = 0; i < size; i++) {
      for (int j = 0; j < size; j++) {
        prec_rec[i][0] += confusionMatrix[i][j];
        prec_rec[j][1] += confusionMatrix[i][j];
      } // for
    } // for

    for (int i = 0; i < size; i++) {
      prec_rec[i][0] = ((double) confusionMatrix[i][i] / prec_rec[i][0]);
      prec_rec[i][1] = ((double) confusionMatrix[i][i] / prec_rec[i][1]);
      prec_rec[i][2] = ((2 * prec_rec[i][0] * prec_rec[i][1]) / (prec_rec[i][0] + prec_rec[i][1]));
    } // for

    return prec_rec;
  } // computePrecisionRecall

  public static void showPrecisionRecall(double [][] prec_rec, int size) {
    System.err.println("Precision\tRecall\t\tF-measure");
    System.err.println("================================================");
    for (int i = 0; i < size; i++) {
      System.err.print(round(prec_rec[i][1], 4) + "\t\t");
      System.err.print(round(prec_rec[i][0], 4) + "\t\t");
      System.err.print(round(prec_rec[i][2], 4) + "\t\t");
      if (i == 0) {
        System.err.println("| -1");
      } // if
      else {
        System.err.println("|  1");
      } // else
    } // for
  } // showPrecisionRecall

  public static void showConfusionMatrix(int [][] confusionMatrix) {
    System.err.println("= Confusion matrix =");
    System.err.println("-1\t1\t");
    System.err.println("====================");
    for (int i = 0; i < 2; i++) {
      for (int j = 0; j < 2; j++) {
        System.err.print(confusionMatrix[i][j] + "\t");
      } // for
      if (i == 0) {
        System.err.println("| -1");
      } // if
      else {
        System.err.println("|  1");
      } // else
    } // for
  } // showConfusionMatrix

  public static void sortFile(String fName) {
    try {
      File f = new File(fName);
      byte []buff = new byte[(int) f.length()];
      FileInputStream fis = new FileInputStream(f);
      fis.read(buff);
      fis.close();
      String text = new String(buff);
      text = removeNoisySpaces(text);

      StringTokenizer strTok = new StringTokenizer(text, "\n");
      String array[] = new String[strTok.countTokens()];
      int i = 0;
      while (strTok.hasMoreTokens()) {
        array[i] = strTok.nextToken();
        i++;
      } // while

      Arrays.sort(array);

      text = "";
      for (i = 0; i < array.length; i++) {
        text += array[i] + "\n";
      } // for

      FileOutputStream fos = new FileOutputStream(f);
      fos.write(text.getBytes());
      fos.close();
    } // try
    catch (Exception e) {
      e.printStackTrace();
    } // catch
  } // sortFile

} // end of class Auxiliary
