/**
 * Greek Named Entity Recognizer.
 * Copyright (C) 2005  G. Lucarelli
 *
 * This file is part of Greek Named Entity Recognizer.
 *
 * Sentence Splitter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sentence Splitter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package ipl.util;

import java.io.*;

import ipl.core.Tokenizer;

/**
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 * @version 0.1
 */
public class FileManager {

  public static String read(String filename) {
    String text = "";

    try {
      File file = new File(filename);

      if (!file.exists() || !file.isFile()) {
        System.err.println("Error reading the file " + filename);
        System.exit(1);
      } // if

      byte buff[] = new byte[(int) file.length()];
      FileInputStream fis = new FileInputStream(file);
      fis.read(buff);
      fis.close();
      text = new String(buff);
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch

    return text;
  } // read

  public static String read(File file) {
    String text = "";

    try {
      if (!file.exists() || !file.isFile()) {
        System.err.println("Error reading the file " + file.getName());
        System.exit(1);
      } // if

      byte buff[] = new byte[(int) file.length()];
      FileInputStream fis = new FileInputStream(file);
      fis.read(buff);
      fis.close();
      text = new String(buff);
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch

    return text;
  } // read

  public static void write(Tokenizer tokens, String filename) {
    String text = Constants.ARTICLE_START_TAG + "\n";
    int size = tokens.size();
    for (int i = 0; i < size; i++) {
      String curr = tokens.get(i).getPrevChar() + tokens.getStartSubtitle(i);
      if (i == 0) {
        curr += Constants.SENTENCE_START_TAG;
      } // if
      else if (tokens.get(i-1).getEndOfPeriod()) {
        curr += Constants.SENTENCE_START_TAG;
      } // else if
      curr += tokens.getStartTag(i) + tokens.get(i).getToken() + tokens.getEndTag(i);
      if (tokens.get(i).getEndOfPeriod()) {
        curr += Constants.SENTENCE_END_TAG;
      } // if
      curr += tokens.getEndSubtitle(i);
      text += curr;
    } // for
    if (!text.endsWith(Constants.SENTENCE_END_TAG)) {
      text += Constants.SENTENCE_END_TAG;
    } // if
    text += "\n" + Constants.ARTICLE_END_TAG;

    try {
      FileOutputStream fos = new FileOutputStream(filename);
      fos.write(text.getBytes());
      fos.close();
    } // try
    catch (IOException e) {
      e.printStackTrace();
    } // catch
  } // write

} // end of class FileManager
